@@ -11,7 +11,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::Compression::Process;
 
@@ -20,6 +20,8 @@ use warnings;
 
 our $VERSION = '1.00';
 
+use Carp;
+
 use Dpkg::Compression;
 use Dpkg::ErrorHandling;
 use Dpkg::Gettext;
@@ -127,8 +129,8 @@ sub _sanity_check {
         $to++ if $opts{"to_$_"};
         $from++ if $opts{"from_$_"};
     }
-    internerr('exactly one to_* parameter is needed') if $to != 1;
-    internerr('exactly one from_* parameter is needed') if $from != 1;
+    croak 'exactly one to_* parameter is needed' if $to != 1;
+    croak 'exactly one from_* parameter is needed' if $from != 1;
     return %opts;
 }
 
@@ -188,7 +190,7 @@ it for you.
 
 sub wait_end_process {
     my ($self, %opts) = @_;
-    $opts{cmdline} ||= $self->{cmdline};
+    $opts{cmdline} //= $self->{cmdline};
     wait_child($self->{pid}, %opts) if $self->{pid};
     delete $self->{pid};
     delete $self->{cmdline};
