@@ -138,7 +138,7 @@ use Dpkg::ErrorHandling;
 
 sub new {
     my $this = shift;
-    my $file = shift || '';
+    my $file = shift // '';
     my $class = ref($this) || $this;
     my $self = {};
     bless $self, $class;
@@ -173,7 +173,7 @@ sub reset {
 sub analyze {
     my ($self, $file) = @_;
 
-    $file ||= $self->{file};
+    $file //= $self->{file};
     return unless $file;
 
     $self->reset;
