@@ -177,9 +177,7 @@ sub add_symbol {
 
     if ($symbol->is_pattern()) {
 	if (my $alias_type = $symbol->get_alias_type()) {
-	    unless (exists $object->{patterns}{aliases}{$alias_type}) {
-		$object->{patterns}{aliases}{$alias_type} = {};
-	    }
+	    $object->{patterns}{aliases}{$alias_type} //= {};
 	    # Alias hash for matching.
 	    my $aliases = $object->{patterns}{aliases}{$alias_type};
 	    $aliases->{$symbol->get_symbolname()} = $symbol;
@@ -282,9 +280,9 @@ sub merge_object_from_symfile {
 
 sub output {
     my ($self, $fh, %opts) = @_;
-    $opts{template_mode} = 0 unless exists $opts{template_mode};
-    $opts{with_deprecated} = 1 unless exists $opts{with_deprecated};
-    $opts{with_pattern_matches} = 0 unless exists $opts{with_pattern_matches};
+    $opts{template_mode} //= 0;
+    $opts{with_deprecated} //= 1;
+    $opts{with_pattern_matches} //= 0;
     my $res = '';
     foreach my $soname (sort $self->get_sonames()) {
 	my @deps = $self->get_dependencies($soname);
