@@ -212,7 +212,7 @@ sub init_options {
     my ($self) = @_;
     # Use full ignore list by default
     # note: this function is not called by V1 packages
-    $self->{options}{diff_ignore_regex} ||= $diff_ignore_default_regex;
+    $self->{options}{diff_ignore_regex} //= $diff_ignore_default_regex;
     $self->{options}{diff_ignore_regex} .= '|(?:^|/)debian/source/local-.*$';
     if (defined $self->{options}{tar_ignore}) {
         $self->{options}{tar_ignore} = [ @tar_ignore_default_pattern ]
@@ -224,7 +224,7 @@ sub init_options {
          'debian/source/local-patch-header';
     # Skip debianization while specific to some formats has an impact
     # on code common to all formats
-    $self->{options}{skip_debianization} ||= 0;
+    $self->{options}{skip_debianization} //= 0;
 
     # Set default compressor for new formats.
     $self->{options}{compression} //= 'xz';
@@ -261,8 +261,7 @@ sub initialize {
 sub upgrade_object_type {
     my ($self, $update_format) = @_;
     $update_format //= 1;
-    $self->{fields}{'Format'} = '1.0'
-        unless exists $self->{fields}{'Format'};
+    $self->{fields}{'Format'} //= '1.0';
     my $format = $self->{fields}{'Format'};
 
     if ($format =~ /^([\d\.]+)(?:\s+\((.*)\))?$/) {
@@ -345,10 +344,9 @@ sub get_basename {
 
 sub find_original_tarballs {
     my ($self, %opts) = @_;
-    $opts{extension} = compression_get_file_extension_regex()
-        unless exists $opts{extension};
-    $opts{include_main} = 1 unless exists $opts{include_main};
-    $opts{include_supplementary} = 1 unless exists $opts{include_supplementary};
+    $opts{extension} //= compression_get_file_extension_regex();
+    $opts{include_main} //= 1;
+    $opts{include_supplementary} //= 1;
     my $basename = $self->get_basename();
     my @tar;
     foreach my $dir ('.', $self->{basedir}, $self->{options}{origtardir}) {
@@ -402,7 +400,7 @@ sub check_signature {
         push @exec, 'gpg', '--no-default-keyring', '-q', '--verify';
     }
     if (scalar(@exec)) {
-        if (defined $ENV{HOME} and -r "$ENV{HOME}/.gnupg/trustedkeys.gpg") {
+        if (length $ENV{HOME} and -r "$ENV{HOME}/.gnupg/trustedkeys.gpg") {
             push @exec, '--keyring', "$ENV{HOME}/.gnupg/trustedkeys.gpg";
         }
         foreach my $vendor_keyring (run_vendor_hook('keyrings')) {
@@ -614,9 +612,7 @@ sub write_dsc {
     }
 
     my $filename = $opts{filename};
-    unless (defined $filename) {
-        $filename = $self->get_basename(1) . '.dsc';
-    }
+    $filename //= $self->get_basename(1) . '.dsc';
     open(my $dsc_fh, '>', $filename)
         or syserr(_g('cannot write %s'), $filename);
     $fields->apply_substvars($opts{substvars});
