@@ -43,7 +43,7 @@ sub create {
     $self->ensure_open('w'); # Creates the file
     *$self->{errors} = 0;
     *$self->{empty} = 1;
-    if ($opts{old} and $opts{new}) {
+    if ($opts{old} and $opts{new} and $opts{filename}) {
         $opts{old} = '/dev/null' unless -e $opts{old};
         $opts{new} = '/dev/null' unless -e $opts{new};
         if (-d $opts{old} and -d $opts{new}) {
@@ -51,7 +51,7 @@ sub create {
         } elsif (-f $opts{old} and -f $opts{new}) {
             $self->add_diff_file($opts{old}, $opts{new}, %opts);
         } else {
-            $self->_fail_not_same_type($opts{old}, $opts{new});
+            $self->_fail_not_same_type($opts{old}, $opts{new}, $opts{filename});
         }
         $self->finish() unless $opts{nofinish};
     }
@@ -64,10 +64,11 @@ sub set_header {
 
 sub add_diff_file {
     my ($self, $old, $new, %opts) = @_;
-    $opts{include_timestamp} = 0 unless exists $opts{include_timestamp};
-    my $handle_binary = $opts{handle_binary_func} || sub {
-        my ($self, $old, $new) = @_;
-        $self->_fail_with_msg($new, _g('binary file contents changed'));
+    $opts{include_timestamp} //= 0;
+    my $handle_binary = $opts{handle_binary_func} // sub {
+        my ($self, $old, $new, %opts) = @_;
+        my $file = $opts{filename};
+        $self->_fail_with_msg($file, _g('binary file contents changed'));
     };
     # Optimization to avoid forking diff if unnecessary
     return 1 if compare($old, $new, 4096) == 0;
@@ -112,7 +113,7 @@ sub add_diff_file {
     while (<$diffgen>) {
         if (m/^(?:binary|[^-+\@ ].*\bdiffer\b)/i) {
             $binary = 1;
-            &$handle_binary($self, $old, $new);
+            &$handle_binary($self, $old, $new, %opts);
             last;
         } elsif (m/^[-+\@ ]/) {
             $difflinefound++;
@@ -147,7 +148,7 @@ sub add_diff_directory {
     # TODO: make this function more configurable
     # - offer to disable some checks
     my $basedir = $opts{basedirname} || basename($new);
-    my $inc_removal = $opts{include_removal} || 0;
+    my $inc_removal = $opts{include_removal} // 0;
     my $diff_ignore;
     if ($opts{diff_ignore_func}) {
         $diff_ignore = $opts{diff_ignore_func};
@@ -168,7 +169,7 @@ sub add_diff_directory {
         my $size = (lstat(_))[7];
         if (-l _) {
             unless (-l "$old/$fn") {
-                $self->_fail_not_same_type("$old/$fn", "$new/$fn");
+                $self->_fail_not_same_type("$old/$fn", "$new/$fn", $fn);
                 return;
             }
             my $n = readlink("$new/$fn");
@@ -180,7 +181,7 @@ sub add_diff_directory {
                 syserr(_g('cannot read link %s'), "$old/$fn");
             }
             unless ($n eq $n2) {
-                $self->_fail_not_same_type("$old/$fn", "$new/$fn");
+                $self->_fail_not_same_type("$old/$fn", "$new/$fn", $fn);
             }
         } elsif (-f _) {
             my $old_file = "$old/$fn";
@@ -190,7 +191,7 @@ sub add_diff_directory {
                 }
                 $old_file = '/dev/null';
             } elsif (not -f _) {
-                $self->_fail_not_same_type("$old/$fn", "$new/$fn");
+                $self->_fail_not_same_type("$old/$fn", "$new/$fn", $fn);
                 return;
             }
 
@@ -202,7 +203,7 @@ sub add_diff_directory {
                                $label_old, "$basedir/$fn"];
         } elsif (-p _) {
             unless (-p "$old/$fn") {
-                $self->_fail_not_same_type("$old/$fn", "$new/$fn");
+                $self->_fail_not_same_type("$old/$fn", "$new/$fn", $fn);
             }
         } elsif (-b _ || -c _ || -S _) {
             $self->_fail_with_msg("$new/$fn",
@@ -213,7 +214,7 @@ sub add_diff_directory {
                     syserr(_g('cannot stat file %s'), "$old/$fn");
                 }
             } elsif (not -d _) {
-                $self->_fail_not_same_type("$old/$fn", "$new/$fn");
+                $self->_fail_not_same_type("$old/$fn", "$new/$fn", $fn);
             }
         } else {
             $self->_fail_with_msg("$new/$fn", _g('unknown file type'));
@@ -236,7 +237,7 @@ sub add_diff_directory {
         } elsif (-l _) {
             warning(_g('ignoring deletion of symlink %s'), $fn);
         } else {
-            $self->_fail_not_same_type("$old/$fn", "$new/$fn");
+            $self->_fail_not_same_type("$old/$fn", "$new/$fn", $fn);
         }
     };
 
@@ -261,7 +262,7 @@ sub add_diff_directory {
         # being shuffled when they are regenerated.
         foreach my $diff_file (sort { $a->[0] cmp $b->[0] } @diff_files) {
             my $fn = $diff_file->[0];
-            $patchorder{$fn} = $i++ unless exists $patchorder{$fn};
+            $patchorder{$fn} //= $i++;
         }
         @diff_files = sort { $patchorder{$a->[0]} <=> $patchorder{$b->[0]} }
                       @diff_files;
@@ -273,6 +274,7 @@ sub add_diff_directory {
         my ($fn, $mode, $size,
             $old_file, $new_file, $label_old, $label_new) = @$diff_file;
         my $success = $self->add_diff_file($old_file, $new_file,
+                                           filename => $fn,
                                            label_old => $label_old,
                                            label_new => $label_new, %opts);
         if ($success and
@@ -311,10 +313,10 @@ sub _fail_with_msg {
     $self->register_error();
 }
 sub _fail_not_same_type {
-    my ($self, $old, $new) = @_;
+    my ($self, $old, $new, $file) = @_;
     my $old_type = get_type($old);
     my $new_type = get_type($new);
-    errormsg(_g('cannot represent change to %s:'), $new);
+    errormsg(_g('cannot represent change to %s:'), $file);
     errormsg(_g('  new version is %s'), $new_type);
     errormsg(_g('  old version is %s'), $old_type);
     $self->register_error();
@@ -403,7 +405,10 @@ sub analyze {
   HUNK:
     while (defined($_) or not eof($self)) {
 	my (%path, %fn);
-	# skip comments leading up to patch (if any)
+
+	# Skip comments leading up to the patch (if any). Although we do not
+	# look for an Index: pseudo-header in the comments, because we would
+	# not use it anyway, as we require both ---/+++ filename headers.
 	while (1) {
 	    if (/^(?:--- |\+\+\+ |@@ -)/) {
 		last;
@@ -543,12 +548,12 @@ sub prepare_apply {
 sub apply {
     my ($self, $destdir, %opts) = @_;
     # Set default values to options
-    $opts{force_timestamp} = 1 unless exists $opts{force_timestamp};
-    $opts{remove_backup} = 1 unless exists $opts{remove_backup};
-    $opts{create_dirs} = 1 unless exists $opts{create_dirs};
-    $opts{options} ||= [ '-t', '-F', '0', '-N', '-p1', '-u',
+    $opts{force_timestamp} //= 1;
+    $opts{remove_backup} //= 1;
+    $opts{create_dirs} //= 1;
+    $opts{options} //= [ '-t', '-F', '0', '-N', '-p1', '-u',
             '-V', 'never', '-g0', '-b', '-z', '.dpkg-orig'];
-    $opts{add_options} ||= [];
+    $opts{add_options} //= [];
     push @{$opts{options}}, @{$opts{add_options}};
     # Check the diff and create missing directories
     my $analysis = $self->analyze($destdir, %opts);
@@ -578,7 +583,7 @@ sub apply {
     # and remove .dpkg-orig files
     my @files = keys %{$analysis->{filepatched}};
     my $now = $opts{timestamp};
-    $now ||= fs_time($files[0]) if $opts{force_timestamp} && scalar @files;
+    $now //= fs_time($files[0]) if $opts{force_timestamp} && scalar @files;
     foreach my $fn (@files) {
 	if ($opts{force_timestamp}) {
 	    utime($now, $now, $fn) or $! == ENOENT
@@ -596,10 +601,10 @@ sub apply {
 sub check_apply {
     my ($self, $destdir, %opts) = @_;
     # Set default values to options
-    $opts{create_dirs} = 1 unless exists $opts{create_dirs};
-    $opts{options} ||= [ '--dry-run', '-s', '-t', '-F', '0', '-N', '-p1', '-u',
+    $opts{create_dirs} //= 1;
+    $opts{options} //= [ '--dry-run', '-s', '-t', '-F', '0', '-N', '-p1', '-u',
             '-V', 'never', '-g0', '-b', '-z', '.dpkg-orig'];
-    $opts{add_options} ||= [];
+    $opts{add_options} //= [];
     push @{$opts{options}}, @{$opts{add_options}};
     # Check the diff and create missing directories
     my $analysis = $self->analyze($destdir, %opts);
