@@ -56,7 +56,7 @@ sub run_hook {
         my ($textref, $ch_info) = @params;
 	if ($ch_info->{'Closes'}) {
 	    foreach my $bug (split(/\s+/, $ch_info->{'Closes'})) {
-		$$textref .= "Bug-Debian: http://bugs.debian.org/$bug\n";
+		$$textref .= "Bug-Debian: https://bugs.debian.org/$bug\n";
 	    }
 	}
 
@@ -73,6 +73,31 @@ sub run_hook {
     }
 }
 
+sub _parse_feature_area {
+    my ($self, $area, $use_feature) = @_;
+
+    # Adjust features based on Maintainer's desires.
+    my $opts = Dpkg::BuildOptions->new(envvar => 'DEB_BUILD_MAINT_OPTIONS');
+    foreach my $feature (split(/,/, $opts->get($area) // '')) {
+	$feature = lc($feature);
+	if ($feature =~ s/^([+-])//) {
+	    my $value = ($1 eq '+') ? 1 : 0;
+	    if ($feature eq 'all') {
+		$use_feature->{$_} = $value foreach keys %{$use_feature};
+	    } else {
+		if (exists $use_feature->{$feature}) {
+		    $use_feature->{$feature} = $value;
+		} else {
+		    warning(_g('unknown %s feature: %s'), $area, $feature);
+		}
+	    }
+	} else {
+	    warning(_g('incorrect value in %s option of ' .
+	               'DEB_BUILD_MAINT_OPTIONS: %s'), $area, $feature);
+	}
+    }
+}
+
 sub add_hardening_flags {
     my ($self, $flags) = @_;
     my $arch = get_host_arch();
@@ -87,6 +112,7 @@ sub add_hardening_flags {
     my %use_feature = (
 	pie => 0,
 	stackprotector => 1,
+	stackprotectorstrong => 1,
 	fortify => 1,
 	format => 1,
 	relro => 1,
@@ -94,25 +120,7 @@ sub add_hardening_flags {
     );
 
     # Adjust features based on Maintainer's desires.
-    my $opts = Dpkg::BuildOptions->new(envvar => 'DEB_BUILD_MAINT_OPTIONS');
-    foreach my $feature (split(/,/, $opts->get('hardening') // '')) {
-	$feature = lc($feature);
-	if ($feature =~ s/^([+-])//) {
-	    my $value = ($1 eq '+') ? 1 : 0;
-	    if ($feature eq 'all') {
-		$use_feature{$_} = $value foreach keys %use_feature;
-	    } else {
-		if (exists $use_feature{$feature}) {
-		    $use_feature{$feature} = $value;
-		} else {
-		    warning(_g('unknown hardening feature: %s'), $feature);
-		}
-	    }
-	} else {
-	    warning(_g('incorrect value in hardening option of ' .
-	               'DEB_BUILD_MAINT_OPTIONS: %s'), $feature);
-	}
-    }
+    $self->_parse_feature_area('hardening', \%use_feature);
 
     # Mask features that are not available on certain architectures.
     if ($os !~ /^(?:linux|knetbsd|hurd)$/ or
@@ -122,13 +130,19 @@ sub add_hardening_flags {
 	#  (#574716).
 	$use_feature{pie} = 0;
     }
-    if ($cpu =~ /^(?:ia64|alpha|mips|mipsel|hppa|arm64)$/ or $arch eq 'arm') {
-	# Stack protector disabled on ia64, alpha, arm64, mips, mipsel, hppa.
+    if ($cpu =~ /^(?:ia64|alpha|mips|mipsel|hppa)$/ or $arch eq 'arm') {
+	# Stack protector disabled on ia64, alpha, mips, mipsel, hppa.
 	#   "warning: -fstack-protector not supported for this target"
 	# Stack protector disabled on arm (ok on armel).
 	#   compiler supports it incorrectly (leads to SEGV)
 	$use_feature{stackprotector} = 0;
     }
+    if ($arch =~ /^(?:m68k|or1k|powerpcspe|sh4|x32)$/) {
+	# "Strong" stack protector disabled on m68k, or1k, powerpcspe, sh4, x32.
+	#   It requires GCC 4.9 and these archs are still using 4.8 as of
+	#   gcc-defaults 1.128.
+	$use_feature{stackprotectorstrong} = 0;
+    }
     if ($cpu =~ /^(?:ia64|hppa|avr32)$/) {
 	# relro not implemented on ia64, hppa, avr32.
 	$use_feature{relro} = 0;
@@ -146,28 +160,43 @@ sub add_hardening_flags {
 	# hardening ability without relro and may incur load penalties.
 	$use_feature{bindnow} = 0;
     }
+    if ($use_feature{stackprotector} == 0) {
+	# Disable stackprotectorstrong if stackprotector is disabled.
+	$use_feature{stackprotectorstrong} = 0;
+    }
 
     # PIE
     if ($use_feature{pie}) {
-	$flags->append('CFLAGS', '-fPIE');
-	$flags->append('OBJCFLAGS', '-fPIE');
-	$flags->append('OBJCXXFLAGS', '-fPIE');
-	$flags->append('FFLAGS', '-fPIE');
-	$flags->append('FCFLAGS', '-fPIE');
-	$flags->append('CXXFLAGS', '-fPIE');
-	$flags->append('GCJFLAGS', '-fPIE');
+	my $flag = '-fPIE';
+	$flags->append('CFLAGS', $flag);
+	$flags->append('OBJCFLAGS',  $flag);
+	$flags->append('OBJCXXFLAGS', $flag);
+	$flags->append('FFLAGS', $flag);
+	$flags->append('FCFLAGS', $flag);
+	$flags->append('CXXFLAGS', $flag);
+	$flags->append('GCJFLAGS', $flag);
 	$flags->append('LDFLAGS', '-fPIE -pie');
     }
 
     # Stack protector
-    if ($use_feature{stackprotector}) {
-	$flags->append('CFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
-	$flags->append('OBJCFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
-	$flags->append('OBJCXXFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
-	$flags->append('FFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
-	$flags->append('FCFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
-	$flags->append('CXXFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
-	$flags->append('GCJFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
+    if ($use_feature{stackprotectorstrong}) {
+	my $flag = '-fstack-protector-strong';
+	$flags->append('CFLAGS', $flag);
+	$flags->append('OBJCFLAGS', $flag);
+	$flags->append('OBJCXXFLAGS', $flag);
+	$flags->append('FFLAGS', $flag);
+	$flags->append('FCFLAGS', $flag);
+	$flags->append('CXXFLAGS', $flag);
+	$flags->append('GCJFLAGS', $flag);
+    } elsif ($use_feature{stackprotector}) {
+	my $flag = '-fstack-protector --param=ssp-buffer-size=4';
+	$flags->append('CFLAGS', $flag);
+	$flags->append('OBJCFLAGS', $flag);
+	$flags->append('OBJCXXFLAGS', $flag);
+	$flags->append('FFLAGS', $flag);
+	$flags->append('FCFLAGS', $flag);
+	$flags->append('CXXFLAGS', $flag);
+	$flags->append('GCJFLAGS', $flag);
     }
 
     # Fortify Source
@@ -177,10 +206,11 @@ sub add_hardening_flags {
 
     # Format Security
     if ($use_feature{format}) {
-	$flags->append('CFLAGS', '-Wformat -Werror=format-security');
-	$flags->append('CXXFLAGS', '-Wformat -Werror=format-security');
-	$flags->append('OBJCFLAGS', '-Wformat -Werror=format-security');
-	$flags->append('OBJCXXFLAGS', '-Wformat -Werror=format-security');
+	my $flag = '-Wformat -Werror=format-security';
+	$flags->append('CFLAGS', $flag);
+	$flags->append('CXXFLAGS', $flag);
+	$flags->append('OBJCFLAGS', $flag);
+	$flags->append('OBJCXXFLAGS', $flag);
     }
 
     # Read-only Relocations
