@@ -3,7 +3,7 @@
 # dpkg-distaddfile
 #
 # Copyright © 1996 Ian Jackson
-# Copyright © 2006-2008,2010,2012 Guillem Jover <guillem@debian.org>
+# Copyright © 2006-2008,2010,2012-2014 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 use strict;
 use warnings;
@@ -26,6 +26,7 @@ use Dpkg ();
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
 use Dpkg::File;
+use Dpkg::Dist::Files;
 
 textdomain('dpkg-dev');
 
@@ -54,9 +55,9 @@ Options:
 
 while (@ARGV && $ARGV[0] =~ m/^-/) {
     $_=shift(@ARGV);
-    if (m/^-f/) {
-        $fileslistfile= $';
-    } elsif (m/^-(\?|-help)$/) {
+    if (m/^-f/p) {
+        $fileslistfile = ${^POSTMATCH};
+    } elsif (m/^-(?:\?|-help)$/) {
         usage();
         exit(0);
     } elsif (m/^--version$/) {
@@ -68,8 +69,8 @@ while (@ARGV && $ARGV[0] =~ m/^-/) {
         usageerr(_g("unknown option \`%s'"), $_);
     }
 }
+usageerr(_g('need exactly a filename, section and priority')) if @ARGV != 3;
 
-@ARGV == 3 || usageerr(_g('need exactly a filename, section and priority'));
 my ($file, $section, $priority) = @ARGV;
 
 ($file =~ m/\s/ || $section =~ m/\s/ || $priority =~ m/\s/) &&
@@ -78,29 +79,18 @@ my ($file, $section, $priority) = @ARGV;
 # Obtain a lock on debian/control to avoid simultaneous updates
 # of debian/files when parallel building is in use
 my $lockfh;
-sysopen($lockfh, 'debian/control', O_WRONLY) ||
-    syserr(_g('cannot write %s'), 'debian/control');
-file_lock($lockfh, 'debian/control');
+my $lockfile = 'debian/control';
+sysopen($lockfh, $lockfile, O_WRONLY)
+    or syserr(_g('cannot write %s'), $lockfile);
+file_lock($lockfh, $lockfile);
 
-$fileslistfile="./$fileslistfile" if $fileslistfile =~ m/^\s/;
-open(my $fileslistnew_fh, '>', "$fileslistfile.new") ||
-    syserr(_g('open new files list file'));
-if (open(my $fileslist_fh, '<', $fileslistfile)) {
-    while (<$fileslist_fh>) {
-        s/\n$//;
-        next if m/^(\S+) / && $1 eq $file;
-        print($fileslistnew_fh "$_\n") ||
-            syserr(_g('copy old entry to new files list file'));
-    }
-    close $fileslist_fh or syserr(_g('cannot close %s'), $fileslistfile);
-} elsif ($! != ENOENT) {
-    syserr(_g('read old files list file'));
-}
-print($fileslistnew_fh "$file $section $priority\n")
-    || syserr(_g('write new entry to new files list file'));
-close($fileslistnew_fh) || syserr(_g('close new files list file'));
-rename("$fileslistfile.new", $fileslistfile) ||
-    syserr(_g('install new files list file'));
+my $dist = Dpkg::Dist::Files->new();
+$dist->load($fileslistfile) if -e $fileslistfile;
+$dist->add_file($file, $section, $priority);
+$dist->save("$fileslistfile.new");
+
+rename("$fileslistfile.new", $fileslistfile)
+    or syserr(_g('install new files list file'));
 
 # Release the lock
-close($lockfh) || syserr(_g('cannot close %s'), 'debian/control');
+close($lockfh) or syserr(_g('cannot close %s'), $lockfile);
