@@ -77,15 +77,18 @@ my $since;
 
 my $substvars_loaded = 0;
 my $substvars = Dpkg::Substvars->new();
-$substvars->set('Format', $changes_format);
+$substvars->set_as_auto('Format', $changes_format);
 
 use constant BUILD_SOURCE     => 1;
 use constant BUILD_ARCH_DEP   => 2;
 use constant BUILD_ARCH_INDEP => 4;
 use constant BUILD_BINARY     => BUILD_ARCH_DEP | BUILD_ARCH_INDEP;
+use constant BUILD_SOURCE_DEP => BUILD_SOURCE | BUILD_ARCH_DEP;
+use constant BUILD_SOURCE_INDEP => BUILD_SOURCE | BUILD_ARCH_INDEP;
 use constant BUILD_ALL        => BUILD_BINARY | BUILD_SOURCE;
 my $include = BUILD_ALL;
 
+sub build_is_default() { return ($include & BUILD_ALL) == BUILD_ALL; }
 sub is_sourceonly() { return $include == BUILD_SOURCE; }
 sub is_binaryonly() { return !($include & BUILD_SOURCE); }
 sub build_opt {
@@ -97,11 +100,24 @@ sub build_opt {
         return '-A';
     } elsif ($include == BUILD_SOURCE) {
         return '-S';
+    } elsif ($include == BUILD_SOURCE_DEP) {
+        return '-G';
+    } elsif ($include == BUILD_SOURCE_INDEP) {
+        return '-g';
     } else {
         croak "build_opt called with include=$include";
     }
 }
 
+sub set_build_type
+{
+    my ($build_type, $build_option) = @_;
+
+    usageerr(_g('cannot combine %s and %s'), build_opt(), $build_option)
+        if not build_is_default and $include != $build_type;
+    $include = $build_type;
+}
+
 sub version {
     printf _g("Debian %s version %s.\n"), $Dpkg::PROGNAME, $Dpkg::PROGVERSION;
 
@@ -116,10 +132,12 @@ sub usage {
 'Usage: %s [<option>...]')
     . "\n\n" . _g(
 "Options:
-  -b                       binary-only build - no source files.
-  -B                       arch-specific - no source or arch-indep files.
-  -A                       only arch-indep - no source or arch-specific files.
-  -S                       source-only upload.
+  -g                       source and arch-indep build.
+  -G                       source and arch-specific build.
+  -b                       binary-only, no source files.
+  -B                       binary-only, only arch-specific files.
+  -A                       binary-only, only arch-indep files.
+  -S                       source-only, no binary files.
   -c<control-file>         get control info from this file.
   -l<changelog-file>       get per-version info from this file.
   -f<files-list-file>      get .deb files list from this file.
@@ -128,8 +146,8 @@ sub usage {
   -m<maintainer>           override control's maintainer value.
   -e<maintainer>           override changelog's maintainer value.
   -u<upload-files-dir>     directory with files (default is '..').
-  -si (default)            src includes orig if new upstream.
-  -sa                      source includes orig src.
+  -si (default)            source includes orig, if new upstream.
+  -sa                      source includes orig, always.
   -sd                      source is diff and .dsc only.
   -q                       quiet - no informational messages on stderr.
   -F<changelog-format>     force changelog format.
@@ -146,23 +164,17 @@ sub usage {
 while (@ARGV) {
     $_=shift(@ARGV);
     if (m/^-b$/) {
-	usageerr(_g('cannot combine %s and %s'), build_opt(), $_)
-	    if is_sourceonly;
-	$include = BUILD_BINARY;
+	set_build_type(BUILD_BINARY, $_);
     } elsif (m/^-B$/) {
-	usageerr(_g('cannot combine %s and %s'), build_opt(), $_)
-	    if is_sourceonly;
-	$include = BUILD_ARCH_DEP;
-	printf { *STDERR } _g('%s: arch-specific upload - not including arch-independent packages') . "\n", $Dpkg::PROGNAME;
+	set_build_type(BUILD_ARCH_DEP, $_);
     } elsif (m/^-A$/) {
-	usageerr(_g('cannot combine %s and %s'), build_opt(), $_)
-	    if is_sourceonly;
-	$include = BUILD_ARCH_INDEP;
-	printf { *STDERR } _g('%s: arch-indep upload - not including arch-specific packages') . "\n", $Dpkg::PROGNAME;
+	set_build_type(BUILD_ARCH_INDEP, $_);
     } elsif (m/^-S$/) {
-	usageerr(_g('cannot combine %s and %s'), build_opt(), $_)
-	    if is_binaryonly;
-	$include = BUILD_SOURCE;
+	set_build_type(BUILD_SOURCE, $_);
+    } elsif (m/^-G$/) {
+	set_build_type(BUILD_SOURCE_DEP, $_);
+    } elsif (m/^-g$/) {
+	set_build_type(BUILD_SOURCE_INDEP, $_);
     } elsif (m/^-s([iad])$/) {
         $sourcestyle= $1;
     } elsif (m/^-q$/) {
@@ -249,7 +261,7 @@ if (not is_sourceonly) {
 
     foreach my $file ($dist->get_files()) {
         if (defined $file->{package} && $file->{package_type} =~ m/^u?deb$/) {
-            $p2f{$file->{package}} ||= [];
+            $p2f{$file->{package}} //= [];
             push @{$p2f{$file->{package}}}, $file->{filename};
         }
 
@@ -276,7 +288,7 @@ foreach (keys %{$src_fields}) {
 # Scan control info of all binary packages
 foreach my $pkg ($control->get_packages()) {
     my $p = $pkg->{'Package'};
-    my $a = $pkg->{'Architecture'} || '';
+    my $a = $pkg->{'Architecture'} // '';
     my $d = $pkg->{'Description'} || 'no description available';
     $d = $1 if $d =~ /^(.*)\n/;
     my $pkg_type = $pkg->{'Package-Type'} ||
@@ -436,8 +448,14 @@ if (!is_binaryonly) {
     for my $f ($checksums->get_files()) {
         $dist->add_file($f, $sec, $pri);
     }
+} elsif ($include == BUILD_ARCH_DEP) {
+    $origsrcmsg = _g('binary-only arch-specific upload ' .
+                     '(source code and arch-indep packages not included)');
+} elsif ($include == BUILD_ARCH_INDEP) {
+    $origsrcmsg = _g('binary-only arch-indep upload ' .
+                     '(source code and arch-specific packages not included)');
 } else {
-    $origsrcmsg= _g('binary-only upload - not including any source code');
+    $origsrcmsg = _g('binary-only upload (no source code included)');
 }
 
 print { *STDERR } "$Dpkg::PROGNAME: $origsrcmsg\n"
@@ -461,6 +479,8 @@ unshift @archvalues, 'source' unless is_binaryonly;
 @archvalues = ('all') if $include == BUILD_ARCH_INDEP;
 @archvalues = grep { !debarch_eq('all', $_) } @archvalues
     unless $include & BUILD_ARCH_INDEP;
+@archvalues = grep { !debarch_eq($host_arch, $_) } @archvalues
+    unless $include & BUILD_ARCH_DEP;
 $fields->{'Architecture'} = join ' ', @archvalues;
 
 $fields->{'Built-For-Profiles'} = join ' ', get_build_profiles();
@@ -475,8 +495,8 @@ for my $file ($dist->get_files()) {
     if (defined $file->{package}) {
         my $arch_all = debarch_eq('all', $p2arch{$file->{package}});
 
-        next if ($include == BUILD_ARCH_DEP and $arch_all);
-        next if ($include == BUILD_ARCH_INDEP and not $arch_all);
+        next if (not ($include & BUILD_ARCH_INDEP) and $arch_all);
+        next if (not ($include & BUILD_ARCH_DEP) and not $arch_all);
     }
     my $uf = "$uploadfilesdir/$f";
     $checksums->add_from_file($uf, key => $f);
