@@ -330,6 +330,9 @@ if ($options{opmode} =~ /^(-b|--print-format|--(before|after)-build|--commit)$/)
     $fields->{'Architecture'} = join(' ', @sourcearch);
     $fields->{'Package-List'} = "\n" . join("\n", sort @pkglist);
 
+    # Check if we have a testsuite, and handle manual and automatic values.
+    set_testsuite_field($fields);
+
     # Scan fields of dpkg-parsechangelog
     foreach (keys %{$changelog}) {
         my $v = $changelog->{$_};
@@ -465,6 +468,22 @@ if ($options{opmode} =~ /^(-b|--print-format|--(before|after)-build|--commit)$/)
     exit(0);
 }
 
+sub set_testsuite_field
+{
+    my ($fields) = @_;
+
+    my $testsuite_field = $fields->{'Testsuite'} // '';
+    my %testsuite = map { $_ => 1 } split /\s*,\s*/, $testsuite_field;
+    if (-e "$dir/debian/tests/control") {
+        $testsuite{autopkgtest} = 1;
+    } elsif ($testsuite{autopkgtest}) {
+        warning(_g('%s field contains value %s, but no tests control file %s'),
+                'Testsuite', 'autopkgtest', 'debian/tests/control');
+        delete $testsuite{autopkgtest};
+    }
+    $fields->{'Testsuite'} = join ', ', sort keys %testsuite;
+}
+
 sub setopmode {
     if (defined($options{opmode})) {
 	usageerr(_g('only one of -x, -b or --print-format allowed, and only once'));
@@ -498,6 +517,7 @@ sub usage {
   -c<control-file>         get control info from this file.
   -l<changelog-file>       get per-version info from this file.
   -F<changelog-format>     force changelog format.
+  --format=<source-format> set the format to be used for the source package.
   -V<name>=<value>         set a substitution variable.
   -T<substvars-file>       read variables here.
   -D<field>=<value>        override or add a .dsc field and value.
@@ -515,7 +535,8 @@ sub usage {
 "Extract options:
   --no-copy                don't copy .orig tarballs
   --no-check               don't check signature and checksums before unpacking
-  --require-valid-signature abort if the package doesn't have a valid signature")
+  --require-valid-signature abort if the package doesn't have a valid signature
+  --ignore-bad-version     allow bad source package versions.")
     . "\n\n" . _g(
 'General options:
   -?, --help               show this help message.
